// Copyright (C) 2002 Johan Hoffman and Anders Logg.
// Licensed under the GNU GPL Version 2.

#include "GlobalField.hh"
#include <Display.hh>
#include <Settings.hh>
#include <Grid.hh>
#include <Vector.hh>

//-----------------------------------------------------------------------------
GlobalField::GlobalField(Grid *grid, real constant)
{
  this->grid = grid;

  no_dof = 1;
  this->nvd = 1;

  representation = CONSTANT;
  dof_values     = 0;
  cell_values    = 0;
  this->constant = constant;
  function       = 0;
}
//-----------------------------------------------------------------------------
GlobalField::GlobalField(Grid *grid, Vector *x, int nvd = 1)
{  
  this->grid = grid;

  no_dof = x->Size()/nvd;
  this->nvd = nvd;

  representation = NODAL;
  dof_values     = x;
  cell_values    = 0;
  constant       = 0.0;
  function       = 0;
}
//-----------------------------------------------------------------------------
GlobalField::GlobalField(Grid *grid, const char *field)
{
  this->grid = grid;
  
  no_dof = 0;
  this->nvd = 1;
  
  representation = FUNCTION;
  dof_values     = 0;
  cell_values    = 0;
  constant       = 0.0;

  function = settings->GetFunction(field);
}
//-----------------------------------------------------------------------------
GlobalField::~GlobalField()
{

  
}
//-----------------------------------------------------------------------------
int GlobalField::GetNoDof()
{
  return no_dof;
}
//-----------------------------------------------------------------------------
int GlobalField::GetVectorDim()
{
  return nvd;
}
//-----------------------------------------------------------------------------
void GlobalField::AttachGrid(Grid* grid)
{
  this->grid = grid;
}
//-----------------------------------------------------------------------------
int GlobalField::GetDim(int cell)
{
  // FIXME: no shapefunctions does not have to be same as no nodes 
  return ( grid->GetCell(cell)->GetSize() );
}
//-----------------------------------------------------------------------------
void GlobalField::GetLocalDof(int cell, real t, real *local_dof, int component = 0)
{
  // FIXME: no shapefunctions does not have to be same as no nodes 
  Cell *c = grid->GetCell(cell);
  int local_dim = c->GetSize();

  switch( representation ){ 
  case NODAL: 
    for (int i=0;i<local_dim;i++)
      local_dof[i] = dof_values->Get(grid->GetCell(cell)->GetNode(i)*nvd + component); 
    break;
  case CELL:
    for (int i=0;i<local_dim;i++) 
      local_dof[i] = cell_values->Get(cell*nvd + component); 
    break;
  case CONSTANT:
    for (int i=0;i<local_dim;i++) 
      local_dof[i] = constant;
    break;
  case FUNCTION:

	 // Return zero if the function is not specified
	 if ( !function ){
		for (int i=0;i<local_dim;i++)
		  local_dof[i] = 0.0;
		return;
	 }

	 // Get the value from the function
	 Point p;
	 for (int i=0;i<local_dim;i++){
		p = grid->GetNode(c->GetNode(i))->GetCoord();
		local_dof[i] = function(p.x,p.y,p.z,t);
	 }
	 
    break;
  case NONE:
    display->InternalError("GlobalField::GetLocalNodes()","GlobalField is not initialized");
  default:
    display->InternalError("GlobalField::GetLocalNodes()","Wrong GlobalField representation");
  }  
}
//-----------------------------------------------------------------------------
real GlobalField::GetConstantValue()
{
  switch( representation ){ 
  case CONSTANT:
    return constant;
    break;
  default:
    display->InternalError("GlobalField::GetLocalNodes()","Not a constant GlobalField");
  }
}
//-----------------------------------------------------------------------------
real GlobalField::GetNodalValue(int node)
{
  return GetNodalValue(node,0);
}
//-----------------------------------------------------------------------------
real GlobalField::GetNodalValue(int node, int component)
{
  return GetDofValue(node,component);
}
//-----------------------------------------------------------------------------
real GlobalField::GetDofValue(int dof)
{
  return GetDofValue(dof,0);
}
//-----------------------------------------------------------------------------
real GlobalField::GetDofValue(int dof, int component)
{
  switch( representation ){ 
  case NODAL: 
    return dof_values->Get(dof*nvd+component);
    break;
  case CONSTANT:
    return constant;
    break;
  default:
    display->InternalError("GlobalField::GetLocalNodes()","GlobalField is not initialized");
  }
}
//-----------------------------------------------------------------------------
real GlobalField::GetCellValue(int cell)
{
  return GetCellValue(cell,0);
}
//-----------------------------------------------------------------------------
real GlobalField::GetCellValue(int cell, int component)
{
  switch( representation ){ 
  case CELL:
    return cell_values->Get(cell*nvd+component);
    break;
  case CONSTANT:
    return constant;
    break;
  default:
    display->InternalError("GlobalField::GetLocalNodes()","GlobalField is not initialized");
  }
}
//-----------------------------------------------------------------------------
